use async_std::{channel::Receiver, task};
use log::{debug, info, warn};

use crate::{db::Database, sbot};

pub enum Task {
    Cancel,
    FetchAll(String),
    //FetchLatest(String),
}

pub async fn spawn(rx: Receiver<Task>, db: Database) {
    task::spawn(async move {
        while let Ok(task) = rx.recv().await {
            match task {
                // Fetch all messages authored by the given peer, filter
                // the root posts and insert them into the peer tree of the
                // database.
                Task::FetchAll(peer) => {
                    let peer_msgs = sbot::get_message_stream(&peer).await;
                    // TODO: return a tuple from sbot::get_root_posts
                    //let (root_posts, latest_sequence) = sbot::get_root_posts(peer_msgs).await;
                    // TODO: update the sequence number if required
                    //if latest_sequence > db.get_latest_sequence(&peer) {
                    //  db.update_sequence(&peer, latest_sequence)
                    //}
                    let root_posts = sbot::get_root_posts(peer_msgs).await;
                    match db.insert_post_batch(&peer, root_posts) {
                        Ok(_) => debug!("inserted batch of posts into post tree for {}", &peer),
                        Err(e) => warn!(
                            "failed to insert batch of posts into post tree for {}: {}",
                            &peer, e
                        ),
                    }
                }
                // TODO: fetch all msgs with sequence number > peer.latest_sequence for peer
                //Task::FetchLatest(peer) => {
                Task::Cancel => {
                    info!("exiting task loop...");
                    break;
                }
            }
        }
    });
}
